#!/usr/bin/env python3

import sys
import argparse
import pandas as pd
from esa import ESA


def create_argparser():
    parser = argparse.ArgumentParser()
    parser.add_argument("--similarity", required = True)
    parser.add_argument("--matrix-path", required = True)
    parser.add_argument("--model-path", required = True)
    parser.add_argument("--model-vocab", required = True)
    parser.add_argument("--text", required = True)
    return parser

def main():
    parser = create_argparser()
    args = parser.parse_args()

    similarity = vars(args)["similarity"]
    matrix_path = vars(args)["matrix_path"]
    model_path = vars(args)["model_path"]
    vocab_path = vars(args)["model_vocab"]
    text = vars(args)["text"]
    e = ESA(matrix_path = matrix_path, model_path = model_path, vocab_path = vocab_path, similarity = similarity)
    result = e.process(text, False)
    print(result)


if __name__ == "__main__":
    main()


import pickle
import gensim
import numpy as np
from scipy import sparse

from argument_esa_model.similarity import Similarity
import sys
from argument_esa_model.preprocessor import Preprocessor
import argument_esa_model.matrix
sys.modules['matrix'] = argument_esa_model.matrix


class ESA:
    def __init__(self, matrix_path, model_path, vocab_path = None, similarity = "cos"):
        self._pre = Preprocessor(vocab_path)
        self._mat = pickle.load(open(matrix_path, "rb"))
        self._mat_2= self._mat.get_mat().transpose().todense()
        self._sim = similarity
        self._model = None
        if similarity != "cos":
            self._model = gensim.models.keyedvectors.KeyedVectors.load_word2vec_format(model_path, binary = True)

    def _to_vector(self, document, lemma):
        if lemma:
            bow = self._pre.to_bow(document, lemma)
            terms_document = tuple(bow.keys())
            length = sum(bow.values())
            vec = sparse.lil_matrix(np.zeros((len(terms_document), 1), dtype = np.longdouble))
            for term in terms_document:
                vec[terms_document.index(term), 0] = bow[term] / length
            vec = sparse.csc_matrix(vec)
            vec.eliminate_zeros()
            return vec, terms_document
        else:
            bow = self._pre.to_bow(document, lemma)
            terms_document = tuple(bow.keys())
            length = sum(bow.values())
            vec = sparse.lil_matrix(np.zeros((len(self._mat.get_terms()), 1), dtype = np.longdouble))
            for term in self._mat.get_terms():
                try:
                    vec[self._mat.get_terms().index(term), 0] = bow[term] / length
                except:
                    vec[self._mat.get_terms().index(term), 0] = 0.0
            vec = sparse.csc_matrix(vec)
            vec.eliminate_zeros()
            return vec, terms_document


    def process(self, document, word_level = False):
        if not word_level:
            return self._process_document(document)
        else:
            raise NotImplementedError
            #return self._process_words(document)


    def _process_document(self, document):
        lemma = False
        if self._sim != "cos":
            lemma = True
        document_vec, terms_document = self._to_vector(document, lemma)
        result = {}

        if self._sim == "cos":
            result_vector= Similarity.cosine_similarity(document_vec, self._mat_2)
            for i,concept in enumerate(self._mat.get_concepts()):
                result[concept] = result_vector[i]

        elif self._sim == "max":
            for concept in self._mat.get_concepts():
                res = Similarity.maximum_matching_similarity(self._model, document_vec, self._mat[concept], terms_document, self._mat._terms)
                result[concept] = res
        elif self._sim == "avg":
                raise NotImplementedError
        return result

'''
    def _process_words(self, document):
        results = {}
        terms = self._mat.get_terms()
        for word in self._pre.to_bow(document):
            if word in terms:
                vec = sparse.lil_matrix(np.zeros((len(terms), 1), dtype = np.longdouble))
                vec[terms.index(word), 0] = 1.0
                vec = sparse.csc_matrix(vec)
                vec.eliminate_zeros()
                intermediate = {}
                for concept in self._mat.get_concepts():
                    intermediate[concept] = self._sim.compute_similarity(vec, self._mat[concept])
                intermediate = sorted(intermediate.items(), key = lambda x : x[1], reverse = True)
                results[word] = intermediate[0][0]
        return results
'''



import re
import pickle
import pandas as pd
from collections import Counter
from nltk.stem import WordNetLemmatizer 

class Preprocessor:


    def __init__(self, vocab_path):
        self._lemmatizer = WordNetLemmatizer()
        if vocab_path is not None:
            self._vocab = set(pickle.load(open(vocab_path,"rb")))


    def preprocess(self, input, lemma = False):
        df = pd.read_csv(input)
        preprocessed = {}
        for index, row in df.iterrows():
            preprocessed[row["concept"]] = self.to_bow(row["text"], lemma)
        return preprocessed


    def tokenize(self, text, lemma):
        text = text.lower()
        text = text.replace("\n", " ").replace("\r", " ")
        text = re.sub(r"https?://(?:[-\w.]|(?:%[\da-fA-F]{2}))+", " ", text)
        tokens = re.sub("[^A-Za-z]+", " ", text).replace("  ", " ")
        if lemma:
            tokens = [self._lemmatizer.lemmatize(word.strip()) for word in tokens.split(" ")]
            tokens = [t for t in tokens if not len(t) < 4]
            tokens = [t for t in tokens if t in self._vocab]
        else:
            tokens = tokens.split(" ")
            tokens = [t for t in tokens if not len(t) < 4]
        return tokens 


    def to_bow(self, text, lemma):
        return dict(Counter(self.tokenize(text, lemma)))



#!/usr/bin/env python3

import pickle

from matrix import ESAMatrix
from preprocessor import Preprocessor


def main():
	for corpus in ["strategic-intelligence", \
		    	"debatepedia",\
                	"wikipedia" \
                	]:
		corpus_path = "../../../../resources/corpora/" + corpus + ".csv"
		matrix_path = "../../../../resources/esa-w2v/" + corpus + ".mat"
		p = Preprocessor()
		preprocessed = p.preprocess(corpus_path, lemma = True)
		concepts = tuple(preprocessed.keys())
		bows = tuple([preprocessed[bow] for bow in preprocessed])
		terms = tuple(set([word for bow in bows for word in bow]))
		print(len(concepts))
		print(len(bows))
		print(len(terms))

		mat = ESAMatrix(terms, concepts, bows)
		pickle.dump(mat, open(matrix_path, "wb"))


if __name__ == "__main__":
	main()



#!/usr/bin/env python3

import argparse
from esa import ESA

from preprocessor import Preprocessor

def create_argparser():
    parser = argparse.ArgumentParser()
    #parser.add_argument("-w", required = False, action = "store_true")
    parser.add_argument("-c", required = True)
    parser.add_argument("-s", required = False, default = "cos", \
                        choices = ["cos", "max", "avg"], \
                        help  = "similarity measure to use")
    parser.add_argument('document', nargs = argparse.REMAINDER)
    return parser


def main():
    parser = create_argparser()
    args = parser.parse_args()

    #word_level = vars(args)["w"]
    word_level = False
    matrix_path = vars(args)["c"]
    similarity = vars(args)["s"]

    model_path = "../../../../resources/esa-w2v/GoogleNews-vectors-negative300.bin"
    matrix_path = "../../../../resources/esa-w2v/debatepedia.mat"
    vocab_path = "../../../../resources/esa-w2v/w2v-vocab.p"
    similarity = "max"

    try:
        input_document = vars(args)["document"][0]

        esa = ESA(matrix_path, model_path, vocab_path, similarity)
        results = esa.process(input_document, word_level)

        if not word_level: 
            for result in results:
                print(f"{results[result]:30.30f} {result}")
        else:
            for result in results:
                print(f"{result} - {results[result]}")
    except IndexError:
        print("ERROR: no input text specified")

  

if __name__ == "__main__":
    main()
